import { parser } from '../grammar/zenuml-parser.js';
import { LRLanguage, LanguageSupport } from '@codemirror/language';
import { styleTags, tags as t } from '@lezer/highlight';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { Compartment } from '@codemirror/state';
const zenumlHighlighting = styleTags({
    TitleKeyword: t.heading,
    TitleContent: t.content,
    // Control flow keywords
    "IfKeyword ElseKeyword WhileKeyword ReturnKeyword NewKeyword TryKeyword CatchKeyword FinallyKeyword": t.keyword,
    // Participant keywords
    "ParticipantKeyword ActorKeyword StereotypeKeyword": t.heading,
    //Structural keywords
    "ParKeyword GroupKeyword OptKeyword CriticalKeyword SectionKeyword FrameKeyword": t.heading,
    // Action keywords
    "MessageKeyword AsyncMessageKeyword ReturnKeyword CreationKeyword": t.heading,
    // Literals
    String: t.string,
    number: t.number,
    "true false null": t.bool,
    // Identifiers
    identifier: t.variableName,
    Type: t.typeName,
    MethodName: t.propertyName,
    // Punctuation
    "OpenBrace CloseBrace": t.brace,
    "OpenParen CloseParen": t.paren,
    "Arrow": t.operator,
    "Dot": t.operator,
    "Comma": t.separator,
    "SemiColon": t.separator,
    // Comments
    Comment: t.lineComment,
    // Special syntax
    Annotation: t.meta,
    Color: t.color,
    Stereotype: t.className,
    Label: t.labelName,
    // Expressions
    UnaryExpression: t.operator,
    BinaryExpression: t.operator,
    FunctionExpression: t.operator,
    InvocationExp: t.operator,
    Parameters: t.operator,
    Parameter: t.operator,
    Declaration: t.operator,
    ArrowOp: t.operator,
    Function: t.operator,
});
// Light theme colors
const lightHighlightStyle = HighlightStyle.define([
    { tag: t.heading, color: "#0d47a1", fontWeight: "bold" },
    { tag: t.content, color: "#1976d2" },
    { tag: t.keyword, color: "#d32f2f" },
    { tag: t.string, color: "#2e7d32" },
    { tag: t.number, color: "#1976d2" },
    { tag: t.bool, color: "#d32f2f" },
    { tag: t.variableName, color: "#0288d1" },
    { tag: t.typeName, color: "#7b1fa2" },
    { tag: t.propertyName, color: "#1565c0" },
    { tag: t.operator, color: "#ed6c02" },
    { tag: t.separator, color: "#757575" },
    { tag: t.lineComment, color: "#757575", fontStyle: "italic" },
    { tag: t.meta, color: "#9c27b0" },
    { tag: t.color, color: "#d32f2f" },
    { tag: t.className, color: "#2e7d32" },
    { tag: t.labelName, color: "#0288d1" }
]);
// Dark theme colors
const darkHighlightStyle = HighlightStyle.define([
    { tag: t.heading, color: "#90caf9", fontWeight: "bold" },
    { tag: t.content, color: "#64b5f6" },
    { tag: t.keyword, color: "#ef5350" },
    { tag: t.string, color: "#81c784" },
    { tag: t.number, color: "#64b5f6" },
    { tag: t.bool, color: "#ef5350" },
    { tag: t.variableName, color: "#4fc3f7" },
    { tag: t.typeName, color: "#ce93d8" },
    { tag: t.propertyName, color: "#64b5f6" },
    { tag: t.operator, color: "#ffb74d" },
    { tag: t.separator, color: "#bdbdbd" },
    { tag: t.lineComment, color: "#bdbdbd", fontStyle: "italic" },
    { tag: t.meta, color: "#e1bee7" },
    { tag: t.color, color: "#ef5350" },
    { tag: t.className, color: "#81c784" },
    { tag: t.labelName, color: "#4fc3f7" }
]);
// Create a compartment for the highlighting style
const highlightStyleCompartment = new Compartment();
export const zenumlLanguage = LRLanguage.define({
    parser: parser.configure({
        props: [zenumlHighlighting]
    }),
    languageData: {
        commentTokens: { line: "//" },
        closeBrackets: { brackets: ["(", "[", "{", "'", '"'] },
        indentOnInput: /^\s*[{}]$/
    }
});
export function zenumlHighlighter() {
    const initialTheme = document.documentElement.getAttribute('data-theme');
    const initialStyle = initialTheme === 'dark' ? darkHighlightStyle : lightHighlightStyle;
    return new LanguageSupport(zenumlLanguage, [
        highlightStyleCompartment.of(syntaxHighlighting(initialStyle))
    ]);
}
// Export a function to update the highlighting style
export function updateHighlightStyle(view, isDark) {
    const style = isDark ? darkHighlightStyle : lightHighlightStyle;
    view.dispatch({
        effects: highlightStyleCompartment.reconfigure(syntaxHighlighting(style))
    });
}
//# sourceMappingURL=zenuml-highlighter.js.map