import { Compartment, EditorState } from '@codemirror/state';
import { EditorView, keymap } from '@codemirror/view';
import { zenumlHighlighter, zenumlCompletions, createThemeExtension, updateHighlightStyle, zenumlLinter, } from './extensions';
import { basicSetup } from 'codemirror';
import { autocompletion, acceptCompletion, closeCompletion, moveCompletionSelection, } from '@codemirror/autocomplete';
import { dracula, tomorrow } from "thememirror";
import { setTheme, getInitialTheme } from './dark-mode';
// Define autocompletion source
// Create a sample document
const doc = `title My Sequence Diagram

participant User
participant System

User -> System: login()
System -> Database: query()
Database -> System: result
System -> User: response

if (isValid) {
	User -> System: success()
} else {
	User -> System: failure()
}`;
const themeCompartment = new Compartment();
const lightTheme = tomorrow;
const darkTheme = dracula;
// Create the editor
const state = EditorState.create({
    doc,
    extensions: [
        basicSetup,
        zenumlHighlighter(),
        autocompletion({
            override: [zenumlCompletions],
            closeOnBlur: true,
            activateOnTyping: true,
            selectOnOpen: true,
            icons: true,
        }),
        themeCompartment.of(lightTheme),
        zenumlLinter,
        keymap.of([
            { key: "Tab", run: acceptCompletion },
            { key: "Enter", run: acceptCompletion },
            { key: "ArrowDown", run: moveCompletionSelection(true) },
            { key: "ArrowUp", run: moveCompletionSelection(false) },
            { key: "Escape", run: closeCompletion }
        ])
    ]
});
// Create and mount the editor
const editorElement = document.getElementById('editor');
// Add event listener for theme toggle
document.addEventListener('DOMContentLoaded', () => {
    const initialTheme = getInitialTheme();
    setTheme(initialTheme);
    if (editorElement) {
        const view = new EditorView({
            state,
            parent: editorElement
        });
        const { updateTheme } = createThemeExtension(view, themeCompartment, darkTheme, lightTheme);
        const themeToggle = document.getElementById('theme-toggle');
        if (themeToggle) {
            themeToggle.addEventListener('click', () => {
                const currentTheme = document.documentElement.getAttribute('data-theme');
                const isDark = currentTheme === 'dark';
                updateTheme(!isDark);
                updateHighlightStyle(view, !isDark);
                setTheme(!isDark ? 'dark' : 'light');
            });
        }
        // Listen for system theme changes
        window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', (e) => {
            if (!localStorage.getItem('theme')) {
                updateTheme(e.matches);
                updateHighlightStyle(view, e.matches);
                setTheme(e.matches ? 'dark' : 'light');
            }
        });
    }
});
//# sourceMappingURL=test.js.map